# src\file_conversor\backend\git_backend.py

"""
This module provides functionalities for handling repositories using Git.
"""

from pathlib import Path
from typing import Any, Callable, Iterable

# user-provided imports
from file_conversor.backend.abstract_backend import AbstractBackend
from file_conversor.backend.http_backend import HttpBackend, NetworkError

from file_conversor.config import Environment, Log, get_translation

from file_conversor.dependency import BrewPackageManager, ScoopPackageManager

from file_conversor.utils.validators import check_file_format

_ = get_translation()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class GitBackend(AbstractBackend):
    """
    GitBackend is a class that provides an interface for handling repositories using Git.
    """

    SUPPORTED_IN_FORMATS = {}

    SUPPORTED_OUT_FORMATS = {}

    EXTERNAL_DEPENDENCIES = {
        "git",
    }

    @staticmethod
    def get_download_url(
        user_name: str,
        repo_name: str,
        file_path: str | Path,
        branch: str = "",
    ):
        """Get the download URL for a file in a GitHub repository.

        :param user_name: The GitHub username or organization name.
        :param repo_name: The name of the repository.
        :param file_path: The path to the file in the repository.        
        :param branch: The branch name. If not provided, the default branch will be used.
        """
        file_path = Path(file_path)
        request_url = f"https://raw.githubusercontent.com/{user_name}/{repo_name}/{branch}/{file_path.as_posix()}"
        return request_url

    @staticmethod
    def get_info_api(
        user_name: str,
        repo_name: str,
        path: str | Path = "",
        branch: str = "",
    ) -> list[dict[str, Any]]:
        """
        Get information about a file or directory in a GitHub repository using the GitHub API.

        return [{
            "name": "filename",
            "path": "path/to/filename",
            "sha": "53098f771f720bf80a8e05ac53f6c281da6cf2b5",
            "size": number | 0 (if a directory),
            "url": "https://api.github.com/repos/<user>/<repo>/contents/.gitmodules?ref=<branch>",
            "html_url": "https://github.com/<user>/<repo>/blob/<branch>/.gitmodules",
            "download_url": "https://raw.githubusercontent.com/<user>/<repo>/<branch>/.gitmodules",
            "type": "file"|"dir",
        }]

        :param user_name: The GitHub username or organization name.
        :param repo_name: The name of the repository.
        :param path: The path to the file or directory in the repository. Defaults to the root directory.
        :param branch: The branch name. If not provided, the default branch will be used.

        :return: A dictionary containing information about the file or directory.

        :raises RuntimeError: if the GitHub API request fails
        """
        http_backend = HttpBackend.get_instance(verbose=False)
        res = http_backend.get_json(
            url=f"https://api.github.com/repos/{user_name}/{repo_name}/contents/{path}",
            params={"ref": branch} if branch else None,
        )
        if isinstance(res, dict) and res.get("status", "200") != "200":
            raise NetworkError(f"{_('Failed to retrieve info from GitHub API')}: {res.get('status', '200')} - {res.get('message', '')}")
        if isinstance(res, dict):
            return [res]
        elif isinstance(res, list):
            return res
        else:
            raise NetworkError(f"{_('Failed to retrieve info from GitHub API')}: {type(res)}")

    @staticmethod
    def check_repository(path: str | Path) -> Path:
        """
        Check if a given path is a Git repository.

        :param path: The path to check.

        :raises FileNotFoundError: if the path does not exist, or is not a .git repository
        """
        repo_path = Path(path).resolve()
        if not (repo_path / ".git").exists():
            raise FileNotFoundError(f"'{repo_path}' is not a git repository")
        return repo_path

    def __init__(
        self,
        install_deps: bool | None,
        verbose: bool = False,
    ):
        """
        Initialize the backend.

        :param install_deps: Install external dependencies. If True auto install using a package manager. If False, do not install external dependencies. If None, asks user for action. 
        :param verbose: Verbose logging. Defaults to False.      

        :raises RuntimeError: if calibre dependency is not found
        """
        super().__init__(
            pkg_managers={
                ScoopPackageManager({
                    "git": "git"
                }),
                BrewPackageManager({
                    "git": "git"
                }),
            },
            install_answer=install_deps,
        )
        self._install_deps = install_deps
        self._verbose = verbose

        # check git
        self._git_bin = self.find_in_path("git")

    def checkout(
        self,
        dest_folder: str | Path,
        branch: str,
    ):
        """
        Checkout a branch in a Git repository.

        :param dest_folder: The destination folder.
        :param branch: The branch to checkout.
        """
        dest_folder = self.check_repository(dest_folder)

        # Execute command
        process = Environment.run(
            str(self._git_bin),
            "checkout",
            branch,
            cwd=dest_folder,
            stdout=None,
            stderr=None,
        )
        return process

    def clone_pull(
        self,
        repo_url: str,
        dest_folder: str | Path,
    ):
        """
        Clone or pull a Git repository.

        :param repo_url: The repository to clone or pull.
        :param dest_folder: The destination folder.
        """
        dest_path = Path(dest_folder).resolve()
        if (dest_path / ".git").exists():
            return self.pull(dest_folder=dest_folder)
        else:
            return self.clone(repo_url=repo_url, dest_folder=dest_folder)

    def clone(
        self,
        repo_url: str,
        dest_folder: str | Path,
    ):
        """
        Clone a Git repository.

        :param repo_url: The repository to clone.
        :param dest_folder: The destination folder.
        """
        dest_path = Path(dest_folder).resolve()
        dest_path.parent.mkdir(parents=True, exist_ok=True)

        if dest_path.exists() and any(dest_path.iterdir()):
            raise FileExistsError(f"'{dest_path}' already exists and is not empty")

        # Execute command
        print(f"{_('This might take a while (couple minutes, or hours) ...')}")

        process = Environment.run(
            str(self._git_bin),
            "clone",
            repo_url,
            str(dest_path),
            stdout=None,
            stderr=None,
        )
        return process

    def pull(
        self,
        dest_folder: str | Path,
    ):
        """
        Pull the latest changes from a Git repository.
        """
        dest_path = self.check_repository(dest_folder)

        # Execute command
        print(f"{_('This might take a while (couple minutes, or hours) ...')}")

        process = Environment.run(
            str(self._git_bin),
            "pull",
            cwd=dest_path,
            stdout=None,
            stderr=None,
        )
        return process


__all__ = [
    "GitBackend",
]



# src/file_conversor/backend/gui/_api/hash/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui._api.hash.check import api_hash_check
from file_conversor.backend.gui._api.hash.create import api_hash_create


def routes():
    return [
        FlaskRoute(
            rule="/api/hash/check",
            handler=api_hash_check,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/hash/create",
            handler=api_hash_create,
            methods=["POST"],
        ),
    ]


__all__ = ['routes']



# src/file_conversor/backend/gui/_components/_class.py


class FlexWrap:
    NO_WRAP = "is-flex-wrap-nowrap"
    WRAP = "is-flex-wrap-wrap"
    WRAP_REVERSE = "is-flex-wrap-wrap-reverse"


class FlexDirection:
    ROW = "is-flex-direction-row"
    ROW_REVERSE = "is-flex-direction-row-reverse"
    COLUMN = "is-flex-direction-column"
    COLUMN_REVERSE = "is-flex-direction-column-reverse"


class FlexJustifyContent:
    FLEX_START = "is-justify-content-flex-start"
    FLEX_END = "is-justify-content-flex-end"
    CENTER = "is-justify-content-center"
    SPACE_BETWEEN = "is-justify-content-space-between"
    SPACE_AROUND = "is-justify-content-space-around"
    SPACE_EVENLY = "is-justify-content-space-evenly"


class FlexAlignItems:
    FLEX_START = "is-align-items-flex-start"
    CENTER = "is-align-items-center"
    FLEX_END = "is-align-items-flex-end"
    SELF_START = "is-align-items-self-start"
    SELF_END = "is-align-items-self-end"
    START = "is-align-items-start"
    END = "is-align-items-end"
    BASELINE = "is-align-items-baseline"
    STRETCH = "is-align-items-stretch"


class FlexAlignContent:
    FLEX_START = "is-align-items-flex-start"
    CENTER = "is-align-items-center"
    FLEX_END = "is-align-items-flex-end"
    START = "is-align-items-start"
    END = "is-align-items-end"
    BASELINE = "is-align-items-baseline"
    STRETCH = "is-align-items-stretch"
    SPACE_AROUND = "is-align-items-space-around"
    SPACE_BETWEEN = "is-align-items-space-between"
    SPACE_EVENLY = "is-align-items-space-evenly"


class FlexAlignSelf:
    AUTO = "is-align-self-auto"
    FLEX_START = "is-align-self-flex-start"
    CENTER = "is-align-self-center"
    FLEX_END = "is-align-self-flex-end"
    START = "is-align-self-start"
    END = "is-align-self-end"
    BASELINE = "is-align-self-baseline"
    STRETCH = "is-align-self-stretch"


class FlexGrow:
    GROW_0 = "is-flex-grow-0"
    GROW_1 = "is-flex-grow-1"
    GROW_2 = "is-flex-grow-2"
    GROW_3 = "is-flex-grow-3"
    GROW_4 = "is-flex-grow-4"
    GROW_5 = "is-flex-grow-5"


class FlexShrink:
    SHRINK_0 = "is-flex-shrink-0"
    SHRINK_1 = "is-flex-shrink-1"
    SHRINK_2 = "is-flex-shrink-2"
    SHRINK_3 = "is-flex-shrink-3"
    SHRINK_4 = "is-flex-shrink-4"
    SHRINK_5 = "is-flex-shrink-5"


__all__ = [
    'FlexWrap',
    'FlexDirection',
    'FlexJustifyContent',
    'FlexAlignItems',
    'FlexAlignContent',
    'FlexAlignSelf',
    'FlexGrow',
    'FlexShrink',
]



# src/file_conversor/backend/gui/_components/margin.py

class MarginAll:
    M_AUTO = "m-auto"
    M_0 = "m-0"
    M_1 = "m-1"
    M_2 = "m-2"
    M_3 = "m-3"
    M_4 = "m-4"
    M_5 = "m-5"
    M_6 = "m-6"


class MarginX:
    M_AUTO = "mx-auto"
    M_0 = "mx-0"
    M_1 = "mx-1"
    M_2 = "mx-2"
    M_3 = "mx-3"
    M_4 = "mx-4"
    M_5 = "mx-5"
    M_6 = "mx-6"


class MarginY:
    M_AUTO = "my-auto"
    M_0 = "my-0"
    M_1 = "my-1"
    M_2 = "my-2"
    M_3 = "my-3"
    M_4 = "my-4"
    M_5 = "my-5"
    M_6 = "my-6"


class MarginLeft:
    M_AUTO = "ml-auto"
    M_0 = "ml-0"
    M_1 = "ml-1"
    M_2 = "ml-2"
    M_3 = "ml-3"
    M_4 = "ml-4"
    M_5 = "ml-5"
    M_6 = "ml-6"


class MarginRight:
    M_AUTO = "mr-auto"
    M_0 = "mr-0"
    M_1 = "mr-1"
    M_2 = "mr-2"
    M_3 = "mr-3"
    M_4 = "mr-4"
    M_5 = "mr-5"
    M_6 = "mr-6"


class MarginTop:
    M_AUTO = "mt-auto"
    M_0 = "mt-0"
    M_1 = "mt-1"
    M_2 = "mt-2"
    M_3 = "mt-3"
    M_4 = "mt-4"
    M_5 = "mt-5"
    M_6 = "mt-6"


class MarginBottom:
    M_AUTO = "mb-auto"
    M_0 = "mb-0"
    M_1 = "mb-1"
    M_2 = "mb-2"
    M_3 = "mb-3"
    M_4 = "mb-4"
    M_5 = "mb-5"
    M_6 = "mb-6"


__all__ = [
    'MarginAll',
    'MarginX',
    'MarginY',
    'MarginTop',
    'MarginBottom',
    'MarginLeft',
    'MarginRight',
]



# src\file_conversor\backend\pdf\__init__.py

from file_conversor.backend.pdf.ghostscript_backend import *
from file_conversor.backend.pdf.ocrmypdf_backend import *
from file_conversor.backend.pdf.pdf2docx_backend import *
from file_conversor.backend.pdf.pikepdf_backend import *
from file_conversor.backend.pdf.pymupdf_backend import *
from file_conversor.backend.pdf.pypdf_backend import *



# src\file_conversor\backend\pymupdf_backend.py

"""
This module provides functionalities for handling files using ``pymupdf`` backend.
"""

import fitz  # pymupdf

from pathlib import Path

from typing import Any, Callable

# user-provided imports
from file_conversor.config import Environment, Log, State
from file_conversor.config.locale import get_translation

from file_conversor.backend.abstract_backend import AbstractBackend

STATE = State.get_instance()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)
_ = get_translation()


class PyMuPDFBackend(AbstractBackend):
    """
    A class that provides an interface for handling files using ``pymupdf``.
    """

    SUPPORTED_IN_FORMATS = {
        "pdf": {},
    }
    SUPPORTED_OUT_FORMATS = {
        "png": {},
        "jpg": {},
    }
    EXTERNAL_DEPENDENCIES: set[str] = set()

    def __init__(
        self,
        verbose: bool = False,
    ):
        """
        Initialize the backend

        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__()
        self._verbose = verbose

    def convert(self,
                output_file: str | Path,
                input_file: str | Path,
                dpi: int = 200,
                password: str | None = None,
                ):
        """
        Convert input files into output file.

        :param output_file: Output file
        :param input_file: Input file. 
        :param dpi: DPI for rendering images. Defaults to 200.
        :param password: Password for encrypted PDF files. Defaults to None.

        :raises FileNotFoundError: if input file not found
        :raises ValueError: if output format is unsupported
        """
        input_file = Path(input_file).resolve()
        output_file = Path(output_file).resolve()

        self.check_file_exists(input_file)

        # open file
        with fitz.open(str(input_file)) as doc:
            if doc.is_encrypted:
                if not password:
                    raise ValueError(_("Password is required for encrypted PDF files"))
                doc.authenticate(password or "")
            # => .png, .jpg, .svg OUTPUT
            for page in doc:
                pix = page.get_pixmap(dpi=dpi)  # type: ignore
                pix.save(f"{output_file.with_suffix("")}_{page.number + 1}{output_file.suffix}")  # type: ignore

    def extract_images(
            self,
            input_file: str | Path,
            output_dir: str | Path,
            progress_callback: Callable[[float], Any] | None = None,
    ):
        """
        Extract all images from a PDF using PyMuPDF (fitz).
        Saves images in their native formats.
        """
        input_file = Path(input_file)
        input_name = input_file.with_suffix("").name

        output_dir = Path(output_dir)

        with fitz.open(input_file) as doc:
            page_len = len(doc)
            for page_index, page in enumerate(doc, start=1):  # type: ignore
                images = page.get_images(full=True)  # list of image xrefs

                for img_index, img in enumerate(images, start=1):
                    xref = img[0]  # xref number of the image
                    base_image = doc.extract_image(xref)

                    img_bytes = base_image["image"]
                    ext = base_image["ext"]  # format: png, jpg, jp2, etc.

                    output_file = output_dir / f"{input_name}_page{page_index}_img{img_index}.{ext}"
                    if not STATE["overwrite-output"] and Path(output_file).exists():
                        raise FileExistsError(f"{_('File')} '{output_file}' {_('exists')}")

                    with open(output_file, "wb") as f:
                        f.write(img_bytes)

                    # logger.debug(f"Extracted {output_file} ({width}x{height})")

                progress = 100.0 * (float(page_index) / page_len)
                if progress_callback:
                    progress_callback(progress)
        if progress_callback:
            progress_callback(100.0)


__all__ = [
    "PyMuPDFBackend",
]



# src\file_conversor\backend\gui\_components\modal.py

from typing import Any

# user-defined imports
from file_conversor.utils.dominate_utils import button, div, header, p, section, footer


def _Modal(**kwargs):
    with div(
        _class="modal",
        **{
            ":class": "{ 'is-active': $store.modal.show }",
            "x-data": "{ }",
        },
        **kwargs,
    ) as modal:

        # Modal background
        div(_class="modal-background")
    return modal


def ModalCard(
        _class_head: str = "",
        _class_body: str = "",
        _class_foot: str = "",
        kwargs_head: dict[str, Any] = {},
        kwargs_body: dict[str, Any] = {},
        kwargs_footer: dict[str, Any] = {},
        **kwargs,
):
    """
    Create a modal card component.

    :param _class_head: Additional CSS classes to apply to the modal header.
    :param _class_body: Additional CSS classes to apply to the modal body.
    :param _class_foot: Additional CSS classes to apply to the modal footer.
    :param kwargs_head: Additional attributes to apply to the modal header.
    :param kwargs_body: Additional attributes to apply to the modal body.
    :param kwargs_footer: Additional attributes to apply to the modal footer.
    :param kwargs: Additional attributes to apply to the modal container.
    """
    with _Modal(**kwargs) as modal:
        # Modal card
        with div(_class="modal-card"):

            # Modal header
            with header(
                _class=f"modal-card-head {_class_head}",
                **kwargs_head,
            ):
                p(
                    "",
                    _class="modal-card-title",
                    **{
                        "x-text": "$store.modal.title",
                    }
                )
                button(
                    _class=f"delete",
                    _aria_label="close",
                    **{
                        ":class": "{ 'is-hidden': !$store.modal.closeable }",
                        "@click": "$store.modal.show = false",
                    },
                )

            # Modal body
            section(
                "",
                _class=f"modal-card-body {_class_body}",
                **kwargs_body,
                **{
                    ":class": "{ 'is-hidden': $store.modal.body == '' }",
                    "x-html": "$store.modal.body",
                }
            )

            # Modal footer
            footer(
                "",
                _class=f"modal-card-foot {_class_foot}",
                **kwargs_footer,
                **{
                    "x-html": "$store.modal.footer",
                }
            )
    return modal


def Modal(**kwargs):
    """
    Create a simple modal component.
    """

    with _Modal(**kwargs) as modal:
        # Modal content
        div(_class="modal-content", **{
            "x-html": "$store.modal.body",
        })
    return modal


__all__ = [
    "ModalCard",
    "Modal",
]



# src\file_conversor\cli\pdf\_info.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

SECURITY_PANEL = _(f"Security commands")
TRANSFORMATION_PANEL = _("Transformations")
OTHERS_PANEL = _("Other commands")

# command
COMMAND_NAME = "pdf"

# SUBCOMMANDS
COMPRESS_NAME = "compress"
CONVERT_NAME = "convert"

DECRYPT_NAME = "decrypt"
ENCRYPT_NAME = "encrypt"

EXTRACT_NAME = "extract"
EXTRACT_IMG_NAME = "extract-img"

MERGE_NAME = "merge"
OCR_NAME = "ocr"
REPAIR_NAME = "repair"

ROTATE_NAME = "rotate"
SPLIT_NAME = "split"

__all__ = [
    "COMMAND_NAME",

    "SECURITY_PANEL",
    "TRANSFORMATION_PANEL",
    "OTHERS_PANEL",

    "COMPRESS_NAME",
    "CONVERT_NAME",

    "DECRYPT_NAME",
    "ENCRYPT_NAME",

    "EXTRACT_NAME",
    "EXTRACT_IMG_NAME",

    "MERGE_NAME",
    "OCR_NAME",
    "REPAIR_NAME",

    "ROTATE_NAME",
    "SPLIT_NAME",
]



from flask import send_from_directory

# user-provided modules
from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.config.environment import Environment

icons_path = Environment.get_icons_folder()


def icons(filename):
    return send_from_directory(icons_path, filename)


def routes():
    return [
        FlaskRoute(
            rule="/icons/<path:filename>",
            handler=icons,
        )
    ]


__all__ = ['routes']



# src/file_conversor/backend/gui/video/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui.video._index import video_index

from file_conversor.backend.gui.video.check import video_check
from file_conversor.backend.gui.video.compress import video_compress
from file_conversor.backend.gui.video.convert import video_convert
from file_conversor.backend.gui.video.enhance import video_enhance
from file_conversor.backend.gui.video.info import video_info
from file_conversor.backend.gui.video.mirror import video_mirror
from file_conversor.backend.gui.video.resize import video_resize
from file_conversor.backend.gui.video.rotate import video_rotate


def routes():
    return [
        FlaskRoute(
            rule="/video",
            handler=video_index,
        ),
        # TOOLS
        FlaskRoute(
            rule="/video/check",
            handler=video_check,
        ),
        FlaskRoute(
            rule="/video/compress",
            handler=video_compress,
        ),
        FlaskRoute(
            rule="/video/convert",
            handler=video_convert,
        ),
        FlaskRoute(
            rule="/video/enhance",
            handler=video_enhance,
        ),
        FlaskRoute(
            rule="/video/info",
            handler=video_info,
        ),
        FlaskRoute(
            rule="/video/mirror",
            handler=video_mirror,
        ),
        FlaskRoute(
            rule="/video/resize",
            handler=video_resize,
        ),
        FlaskRoute(
            rule="/video/rotate",
            handler=video_rotate,
        ),
    ]


__all__ = [
    'video_index',
    'video_check',
    'video_compress',
    'video_convert',
    'video_enhance',
    'video_info',
    'video_mirror',
    'video_resize',
    'video_rotate',

    'routes',
]



# src/file_conversor/backend/gui/_components/grid.py

from typing import Any

from file_conversor.utils.dominate_utils import div


def Cell(*content: Any, _class="", **kwargs):
    """
    Create a grid cell.

    :param content: The content of the cell.
    :param _class: Additional CSS classes for the cell.
    """
    with div(_class=f"cell {_class}", **kwargs) as cell:
        for item in content:
            if not item:
                continue
            cell.add(item)
    return cell


def FixedGrid(
        *cells,
        columns: str = 'auto',
        _class="",
        **kwargs,
):
    """
    Create a fixed grid layout.

    :param cells: The cells to be placed in the grid.
    :param columns: The number of columns in the grid. Options are 'auto', '0' upto '12'.
    :param _class: Additional CSS classes for the grid.
    """
    if columns != 'auto' and int(columns) not in range(0, 13):
        raise ValueError("columns must be 'auto' or an integer between 0 and 12.")
    with div(_class=f"fixed-grid has-{columns}-cols {_class}", **kwargs) as grid:
        with div(_class="grid") as inner_grid:
            for cell in cells:
                if not cell:
                    continue
                inner_grid.add(cell)
    return grid


def SmartGrid(
        *cells,
        _class="",
        **kwargs,
):
    """
    Create a smart grid layout.

    :param cells: The cells to be placed in the grid.
    :param _class: Additional CSS classes for the grid.
    """
    with div(_class=f"grid {_class}", **kwargs) as grid:
        for cell in cells:
            if not cell:
                continue
            grid.add(cell)
    return grid


__all__ = [
    'Cell',
    'SmartGrid',
    'FixedGrid',
]



# src\file_conversor\backend\gui\flask_api_status.py

from flask import Flask
from typing import Any, Callable, Self

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import AVAILABLE_LANGUAGES, get_system_locale, get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


class FlaskApiStatus:
    def __init__(
            self,
            id: str | int,
            status: str = '',
            message: str = '',
            exception: str = '',
            progress: int | None = None,
    ) -> None:
        super().__init__()
        self._id = str(id)
        self._status = status
        self._message = message
        self._exception = exception
        self._progress = progress

    def __repr__(self) -> str:
        return f"FlaskStatus(id={self._id}, status={self._status}, message={self._message}, exception={self._exception}, progress={self._progress})"

    def __str__(self) -> str:
        return self.__repr__()

    def __eq__(self, value) -> bool:
        if not isinstance(value, FlaskApiStatus):
            return False
        return (self._id == value._id and
                self._status == value._status)

    def get_id(self) -> str:
        return self._id

    def get_status(self) -> str:
        return self._status

    def get_message(self) -> str:
        return self._message

    def get_exception(self) -> str:
        return self._exception

    def get_progress(self) -> int | None:
        return self._progress

    def set_progress(self, progress: int | float) -> None:
        self._progress = int(progress)

    def set_message(self, message: str) -> None:
        self._message = message

    def set(self, other: 'FlaskApiStatus') -> None:
        self._status = other._status
        self._message = other._message
        self._exception = other._exception
        self._progress = other._progress

    def json(self) -> dict[str, Any]:
        return {
            'id': self._id,
            'status': self._status,
            'message': self._message,
            'exception': self._exception,
            'progress': self._progress,
        }


class FlaskApiStatusCompleted(FlaskApiStatus):
    def __init__(
            self,
            id: str | int,
            message: str = "",
    ) -> None:
        super().__init__(
            id=id,
            status='completed',
            message=message,
            progress=100,
        )


class FlaskApiStatusProcessing(FlaskApiStatus):
    def __init__(
            self,
            id: str | int,
            progress: int | None = None,
            message: str = "",
    ) -> None:
        super().__init__(
            id=id,
            status='processing',
            message=message,
            progress=progress,
        )


class FlaskApiStatusReady(FlaskApiStatus):
    def __init__(self) -> None:
        super().__init__(
            id='0',
            status='ready',
        )


class FlaskApiStatusError(FlaskApiStatus):
    def __init__(
            self,
            id: str | int,
            exception: str,
            progress: int | None = None,
    ) -> None:
        super().__init__(
            id=id,
            status='failed',
            exception=exception,
            progress=progress,
        )


class FlaskApiStatusUnknown(FlaskApiStatus):
    def __init__(
            self,
            id: str | int,
    ) -> None:
        super().__init__(
            id=id,
            status='unknown',
            exception=_('The provided status ID does not exist.'),
        )


__all__ = [
    'FlaskApiStatus',
    'FlaskApiStatusCompleted',
    'FlaskApiStatusProcessing',
    'FlaskApiStatusReady',
    'FlaskApiStatusError',
    'FlaskApiStatusUnknown',
]



# src\file_conversor\utils\bulma\form_output_file.py

import os

from pathlib import Path
from typing import Any, Sequence

from file_conversor.utils.dominate_bulma.form_input import FormFieldInput

from file_conversor.utils.formatters import format_py_to_js


def FormFieldOutputFile(
    help: str,
    _name: str,
    file_types: Sequence[str],
    _readonly: bool = False,
    x_data: str = "",
    x_init: str = "",
    **kwargs,
):
    """
    Create a form field for selecting an output file.

    :param help: Optional help text.
    :param _name: The name attribute for the select element.
    :param file_types: List of accepted file types (e.g., ['.pdf', '.docx']).
    :param _readonly: Whether the input field is read-only.
    :param x_data: Additional x-data for the component.
    :param x_init: Additional x-init for the component.
    """
    return FormFieldInput(
        validation_expr="value.length > 0",
        _readonly=_readonly,
        _name=_name,
        help=help,
        button={
            "icon": {"name": "file-export"},
            "_class": "ml-2 is-info",
            "_title": help,
            "@click": "saveFileDialog",
        },
        x_data=f"""            
            async saveFileDialog() {{
                const fileList = await pywebview.api.save_file_dialog({{                    
                    filename: this.value,
                    file_types: {format_py_to_js(file_types)},
                }});
                console.log('Save file dialog returned:', fileList);
                if (fileList && fileList.length > 0) {{
                    this.value = fileList[0];
                }}
            }},
            {x_data}
        """,
        x_init=f"""
            let thisObj = this;
            windowEventHandler.on('pywebviewready', async () => {{
                const dir = await pywebview.api.get_last_open_dir();
                if (thisObj.value && thisObj.value !== '') {{
                    thisObj.value = dir + '{os.sep if os.sep == '/' else r'\\'}' + thisObj.value;
                }}
            }});
            {x_init}
        """,
        **kwargs,
    )


__all__ = [
    "FormFieldOutputFile",
]



# src\file_conversor\utils\dominate_bulma\page_card_grid.py

from typing import Any

# user-provided modules
from file_conversor.utils.dominate_bulma.breadcrumb import Breadcrumb
from file_conversor.utils.dominate_bulma.card import Card
from file_conversor.utils.dominate_bulma.figure import Figure
from file_conversor.utils.dominate_bulma.grid import Cell, SmartGrid
from file_conversor.utils.dominate_bulma.media import Media
from file_conversor.utils.dominate_bulma.page_base import PageBase

from file_conversor.utils.dominate_utils import a, div, p

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def PageCardGrid(
        *card_items: dict[str, str],
        nav_items: list[dict[str, Any]],
        _title: str,
        **kwargs,
):
    """
    Create a page with a grid of cards.

    ```python
    card_items = [{
        'image': The URL of the image to display on the card,
        'title': The title text of the card,
        'subtitle': The subtitle text of the card,
        'url': The URL the card links to,
    }]

    nav_items = [{
        'icon': (Optional) The icon name to display alongside the label
        'label': The display text for the item,
        'url': The URL the item links to,
        'active': A boolean indicating if the item is the current page,
    }]
    ```

    :param card_items: The items to display in the card grid. Each item should be a dictionary with 'image', 'title', 'subtitle', and 'url' keys.
    :param nav_items: The breadcrumb navigation items.
    :param _title: The title of the page.
    :param kwargs: Additional keyword arguments for the PageBase.
    """
    return PageBase(
        SmartGrid(
            *[
                Cell(
                    a(
                        Card(
                            Media(
                                left=Figure(_src=item['image'], _class="is-48x48"),
                                content=div(
                                    p(item['title'], _class="title is-4"),
                                    p(item['subtitle'], _class="subtitle is-6"),
                                ),
                            ),
                        ),
                        _href=item['url'],
                    )
                )
                for item in card_items
                if item
            ],
            _class="is-col-min-12 is-gap-2",
        ),
        nav_title=Breadcrumb(*nav_items),
        _title=_title,
        **kwargs,
    )


__all__ = [
    "PageCardGrid",
]



# src\file_conversor\backend\gui\pdf\_dom_page.py

from flask import url_for
from typing import Any

# user-provided modules
from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def pdf_index_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("PDF"),
        'url': url_for('pdf_index'),
        'active': active,
    }


def pdf_compress_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Compress"),
        'url': url_for('pdf_compress'),
        'active': active,
    }


def pdf_convert_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Convert"),
        'url': url_for('pdf_convert'),
        'active': active,
    }


def pdf_decrypt_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Decrypt"),
        'url': url_for('pdf_decrypt'),
        'active': active,
    }


def pdf_encrypt_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Encrypt"),
        'url': url_for('pdf_encrypt'),
        'active': active,
    }


def pdf_extract_img_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Extract Images"),
        'url': url_for('pdf_extract_img'),
        'active': active,
    }


def pdf_extract_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Extract Pages"),
        'url': url_for('pdf_extract'),
        'active': active,
    }


def pdf_merge_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Merge"),
        'url': url_for('pdf_merge'),
        'active': active,
    }


def pdf_ocr_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("OCR"),
        'url': url_for('pdf_ocr'),
        'active': active,
    }


def pdf_repair_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Repair"),
        'url': url_for('pdf_repair'),
        'active': active,
    }


def pdf_rotate_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Rotate"),
        'url': url_for('pdf_rotate'),
        'active': active,
    }


def pdf_split_nav_item(
    active: bool = False
) -> dict[str, Any]:
    return {
        'label': _("Split"),
        'url': url_for('pdf_split'),
        'active': active,
    }


__all__ = [
    'pdf_index_nav_item',
    'pdf_compress_nav_item',
    'pdf_convert_nav_item',
    'pdf_decrypt_nav_item',
    'pdf_encrypt_nav_item',
    'pdf_extract_img_nav_item',
    'pdf_extract_nav_item',
    'pdf_merge_nav_item',
    'pdf_ocr_nav_item',
    'pdf_repair_nav_item',
    'pdf_rotate_nav_item',
    'pdf_split_nav_item',
]



# src/file_conversor/backend/gui/text/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui.text._index import text_index

from file_conversor.backend.gui.text.check import text_check
from file_conversor.backend.gui.text.compress import text_compress
from file_conversor.backend.gui.text.convert import text_convert


def routes():
    return [
        FlaskRoute(
            rule="/text",
            handler=text_index,
        ),
        FlaskRoute(
            rule="/text/check",
            handler=text_check,
        ),
        FlaskRoute(
            rule="/text/compress",
            handler=text_compress,
        ),
        FlaskRoute(
            rule="/text/convert",
            handler=text_convert,
        ),
    ]


__all__ = [
    'text_index',
    'text_check',
    'text_compress',
    'text_convert',

    'routes',
]




# src\file_conversor\cli\pdf\compress_cmd.py

import tempfile
import typer

from pathlib import Path
from typing import Annotated, Any, Callable, List

from rich import print

# user-provided modules
from file_conversor.backend.pdf import PikePDFBackend, GhostscriptBackend

from file_conversor.cli.pdf._typer import TRANSFORMATION_PANEL as RICH_HELP_PANEL
from file_conversor.cli.pdf._typer import COMMAND_NAME, COMPRESS_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import InputFilesArgument, OutputDirOption
from file_conversor.utils.validators import check_valid_options

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = {
    *PikePDFBackend.EXTERNAL_DEPENDENCIES,
    *GhostscriptBackend.EXTERNAL_DEPENDENCIES,
}


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    for ext in GhostscriptBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="compress",
                description="Compress",
                command=f'cmd.exe /c "{Environment.get_executable()} "{COMMAND_NAME}" "{COMPRESS_NAME}" "%1""',
                icon=str(icons_folder_path / 'compress.ico'),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_pdf_compress_cmd(
    input_files: List[Path],
    compression: str,
    output_dir: Path,
    progress_callback: Callable[[float], Any] = lambda p: p,
):
    pikepdf_backend = PikePDFBackend(verbose=STATE["verbose"])
    gs_backend = GhostscriptBackend(
        install_deps=CONFIG['install-deps'],
        verbose=STATE['verbose'],
    )

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        with tempfile.TemporaryDirectory() as temp_dir:
            gs_out = Path(temp_dir) / CommandManager.get_output_file(input_file, stem="_gs")
            gs_backend.compress(
                input_file=input_file,
                output_file=gs_out,
                compression_level=GhostscriptBackend.Compression.from_str(compression),
                progress_callback=lambda p: progress_callback(progress_mgr.update_progress(p))
            )
            progress_callback(progress_mgr.complete_step())

            pikepdf_backend.compress(
                # files
                input_file=gs_out,
                output_file=output_file,
                progress_callback=lambda p: progress_callback(progress_mgr.update_progress(p))
            )
            progress_callback(progress_mgr.complete_step())
            print(f"Processing '{output_file}' ... OK")

    cmd_mgr = CommandManager(input_files, output_dir=output_dir, steps=2, overwrite=STATE["overwrite-output"])
    cmd_mgr.run(callback, out_stem="_compressed")

    logger.info(f"{_('File compression')}: [green][bold]{_('SUCCESS')}[/bold][/green]")


@typer_cmd.command(
    name=COMPRESS_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Reduce the file size of a PDF document (requires Ghostscript external library).')}
        
        {_('Outputs a file with _compressed at the end.')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor {COMMAND_NAME} {COMPRESS_NAME} input_file.pdf -od D:/Downloads`

        - `file_conversor {COMMAND_NAME} {COMPRESS_NAME} input_file.pdf -c high`

        - `file_conversor {COMMAND_NAME} {COMPRESS_NAME} input_file.pdf -o`
    """)
def compress(
    input_files: Annotated[List[Path], InputFilesArgument(GhostscriptBackend)],
    compression: Annotated[str, typer.Option("--compression", "-c",
                                             help=f"{_('Compression level (high compression = low quality). Valid values are')} {', '.join(GhostscriptBackend.Compression.get_dict())}. {_('Defaults to')} {CONFIG["pdf-compression"]}.",
                                             callback=lambda x: check_valid_options(x, GhostscriptBackend.Compression.get_dict()),
                                             )] = CONFIG["pdf-compression"],
    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    execute_pdf_compress_cmd(
        input_files=input_files,
        compression=compression,
        output_dir=output_dir,
    )

    logger.info(f"{_('File compression')}: [green][bold]{_('SUCCESS')}[/bold][/green]")


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
    "execute_pdf_compress_cmd",
]




# src\file_conversor\cli\multimedia\enhance_cmd.py
import typer

from pathlib import Path
from typing import Annotated, Any, Callable, List

from rich import print

# user-provided modules
from file_conversor.backend.image import PillowBackend
from file_conversor.cli.image._typer import FILTER_PANEL as RICH_HELP_PANEL
from file_conversor.cli.image._typer import COMMAND_NAME, ENHANCE_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import BrightnessOption, ColorOption, ContrastOption, InputFilesArgument, OutputDirOption, SharpnessOption
from file_conversor.utils.validators import is_close

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = PillowBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    # IMG2PDF commands
    for ext in PillowBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="enhance",
                description="Enhance",
                command=f'cmd.exe /k "{Environment.get_executable()} "{COMMAND_NAME}" "{ENHANCE_NAME}" "%1""',
                icon=str(icons_folder_path / "color.ico"),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_image_enhance_cmd(
    input_files: List[Path],
    brightness: float,
    contrast: float,
    color: float,
    sharpness: float,
    output_dir: Path,
    progress_callback: Callable[[float], Any] = lambda p: p,
):
    pillow_backend = PillowBackend(verbose=STATE['verbose'])

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        logger.info(f"Processing '{output_file}' ... ")
        pillow_backend.enhance(
            input_file=input_file,
            output_file=output_file,
            color_factor=color,
            brightness_factor=brightness,
            contrast_factor=contrast,
            sharpness_factor=sharpness,
        )
        progress_callback(progress_mgr.complete_step())

    cmd_mgr = CommandManager(input_files, output_dir=output_dir, overwrite=STATE["overwrite-output"])
    cmd_mgr.run(callback, out_stem="_enhanced")

    logger.info(f"{_('Image enhance')}: [green bold]{_('SUCCESS')}[/]")


@typer_cmd.command(
    name=ENHANCE_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Enhance image color, brightness, contrast, or sharpness.')}
    """,
    epilog=f"""
        **{_('Examples')}:**

        - `file_conversor {COMMAND_NAME} {ENHANCE_NAME} input_file.jpg -od D:/Downloads --color 1.20`

        - `file_conversor {COMMAND_NAME} {ENHANCE_NAME} input_file1.bmp --sharpness 0.85`

        - `file_conversor {COMMAND_NAME} {ENHANCE_NAME} input_file.jpg -cl 0.85 -b 1.10`        
    """)
def enhance(
    input_files: Annotated[List[Path], InputFilesArgument(PillowBackend)],

    brightness: Annotated[float, BrightnessOption()] = 1.00,

    contrast: Annotated[float, ContrastOption()] = 1.00,

    color: Annotated[float, ColorOption()] = 1.00,

    sharpness: Annotated[float, SharpnessOption()] = 1.00,

    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    if is_close(brightness, 1.00) and is_close(contrast, 1.00) and is_close(color, 1.00) and is_close(sharpness, 1.00):
        brightness = typer.prompt("Brightness factor (> 1.0 increases, < 1.0 decreases)", default=1.00)
        contrast = typer.prompt("Contrast factor (> 1.0 increases, < 1.0 decreases)", default=1.00)
        color = typer.prompt("Color factor (> 1.0 increases, < 1.0 decreases)", default=1.00)
        sharpness = typer.prompt("Sharpness factor (> 1.0 increases, < 1.0 decreases)", default=1.00)

    execute_image_enhance_cmd(
        input_files=input_files,
        brightness=brightness,
        contrast=contrast,
        color=color,
        sharpness=sharpness,
        output_dir=output_dir,
    )


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
    "execute_image_enhance_cmd",
]




# src\file_conversor\cli\win\inst_menu_cmd.py

import typer

from typing import Annotated

from rich import print


# user-provided modules
from file_conversor.backend import WinRegBackend

import file_conversor.cli.win.uninstall_menu_cmd as uninstall_menu_cmd
import file_conversor.cli.win.restart_explorer_cmd as restart_explorer_cmd

from file_conversor.cli.win._typer import CONTEXT_MENU_PANEL as RICH_HELP_PANEL
from file_conversor.cli.win._typer import COMMAND_NAME, INSTALL_MENU_NAME

from file_conversor.config import Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.system.win.ctx_menu import WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = WinRegBackend.EXTERNAL_DEPENDENCIES


@typer_cmd.command(
    name=INSTALL_MENU_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Installs app context menu (right click in Windows Explorer).')}        
    """,
    epilog=f"""
**{_('Examples')}:** 

- `file_conversor {COMMAND_NAME} {INSTALL_MENU_NAME}` 
""")
def install_menu(
    reboot_explorer: Annotated[bool, typer.Option("--restart-explorer", "-re",
                                                  help=_("Restart explorer.exe (to make ctx menu effective immediately). Defaults to False (do not restart, user must log off/in to make ctx menu changes effective)"),
                                                  is_flag=True,
                                                  )] = False,
):
    winreg_backend = WinRegBackend(verbose=STATE["verbose"])

    uninstall_menu_cmd.uninstall_menu()

    logger.info(f"{_('Installing app context menu in Windows Explorer')} ...")

    # Define registry path
    ctx_menu = WinContextMenu.get_instance()

    winreg_backend.import_file(ctx_menu.get_reg_file())

    if reboot_explorer:
        restart_explorer_cmd.restart_explorer()
    else:
        logger.warning("Restart explorer.exe or log off from Windows, to make changes effective immediately.")

    logger.info(f"{_('Context Menu Install')}: [bold green]{_('SUCCESS')}[/].")


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
]




# src\file_conversor\cli\video\convert_cmd.py

import typer

from rich import print

from typing import Annotated, List
from pathlib import Path

# user-provided modules
from file_conversor.backend import FFmpegBackend

from file_conversor.cli.video._ffmpeg_cmd_helper import FFmpegCmdHelper, EXTERNAL_DEPENDENCIES

from file_conversor.cli.video._typer import TRANSFORMATION_PANEL as RICH_HELP_PANEL
from file_conversor.cli.video._typer import COMMAND_NAME, CONVERT_NAME

from file_conversor.config import Environment, Configuration, State, Log, get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.validators import check_valid_options
from file_conversor.utils.typer_utils import (InputFilesArgument, FormatOption, OutputDirOption,
                                              AudioBitrateOption, VideoBitrateOption,
                                              AudioCodecOption, VideoCodecOption,
                                              AxisOption,
                                              BrightnessOption, ColorOption, ContrastOption, GammaOption,
                                              VideoRotationOption, DeshakeOption, UnsharpOption,
                                              FPSOption, ResolutionOption,
                                              VideoEncodingSpeedOption, VideoQualityOption)

from file_conversor.system.win import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()


def register_ctx_menu(ctx_menu: WinContextMenu):
    # FFMPEG commands
    icons_folder_path = Environment.get_icons_folder()
    for ext in FFmpegBackend.SUPPORTED_IN_VIDEO_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="to_mkv",
                description="To MKV",
                command=f'cmd.exe /c "{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" -f mkv "%1""',
                icon=str(icons_folder_path / 'mkv.ico'),
            ),
            WinContextCommand(
                name="to_mp4",
                description="To MP4",
                command=f'cmd.exe /c "{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" -f mp4 "%1""',
                icon=str(icons_folder_path / 'mp4.ico'),
            ),
            WinContextCommand(
                name="to_webm",
                description="To WEBM",
                command=f'cmd.exe /c "{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" -f webm "%1""',
                icon=str(icons_folder_path / 'webm.ico'),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


@typer_cmd.command(
    name=CONVERT_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Convert a video file to another video format.')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor {COMMAND_NAME} {CONVERT_NAME} input_file.webm -f mkv -od output_dir/ --audio-bitrate 192`

        - `file_conversor {COMMAND_NAME} {CONVERT_NAME} input_file.mp4 -f avi -r 90`

        - `file_conversor {COMMAND_NAME} {CONVERT_NAME} input_file.avi -f mp4 -rs 1280:720`
    """)
def convert(
    input_files: Annotated[List[Path], InputFilesArgument(FFmpegBackend.SUPPORTED_IN_VIDEO_FORMATS)],

    file_format: Annotated[str, FormatOption(FFmpegBackend.SUPPORTED_OUT_VIDEO_FORMATS)],

    audio_bitrate: Annotated[int, AudioBitrateOption()] = CONFIG["audio-bitrate"],
    video_bitrate: Annotated[int, VideoBitrateOption()] = CONFIG["video-bitrate"],

    audio_codec: Annotated[str | None, AudioCodecOption(FFmpegBackend.get_supported_audio_codecs())] = None,
    video_codec: Annotated[str | None, VideoCodecOption(FFmpegBackend.get_supported_video_codecs())] = None,

    video_encoding_speed: Annotated[str | None, VideoEncodingSpeedOption(FFmpegBackend.ENCODING_SPEEDS)] = CONFIG["video-encoding-speed"],
    video_quality: Annotated[str | None, VideoQualityOption(FFmpegBackend.QUALITY_PRESETS)] = CONFIG["video-quality"],

    resolution: Annotated[str | None, ResolutionOption()] = None,
    fps: Annotated[int | None, FPSOption()] = None,

    brightness: Annotated[float, BrightnessOption()] = 1.0,
    contrast: Annotated[float, ContrastOption()] = 1.0,
    color: Annotated[float, ColorOption()] = 1.0,
    gamma: Annotated[float, GammaOption()] = 1.0,

    rotation: Annotated[int | None, VideoRotationOption()] = None,
    mirror_axis: Annotated[str | None, AxisOption()] = None,
    deshake: Annotated[bool, DeshakeOption()] = False,
    unsharp: Annotated[bool, UnsharpOption()] = False,

    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):

    ffmpeg_cmd_helper = FFmpegCmdHelper(
        install_deps=CONFIG['install-deps'],
        verbose=STATE["verbose"],
        overwrite_output=STATE["overwrite-output"],
    )

    # Set arguments for FFmpeg command helper
    ffmpeg_cmd_helper.set_input(input_files)
    ffmpeg_cmd_helper.set_output(file_format=file_format, output_dir=output_dir)

    ffmpeg_cmd_helper.set_video_settings(encoding_speed=video_encoding_speed, quality=video_quality)
    ffmpeg_cmd_helper.set_bitrate(audio_bitrate=audio_bitrate, video_bitrate=video_bitrate)
    ffmpeg_cmd_helper.set_codecs(audio_codec=audio_codec, video_codec=video_codec)

    ffmpeg_cmd_helper.set_resolution_filter(resolution)
    ffmpeg_cmd_helper.set_fps_filter(fps)
    ffmpeg_cmd_helper.set_enhancement_filters(
        brightness=brightness,
        contrast=contrast,
        color=color,
        gamma=gamma,
    )
    ffmpeg_cmd_helper.set_rotation_filter(rotation)
    ffmpeg_cmd_helper.set_mirror_filter(mirror_axis)
    ffmpeg_cmd_helper.set_deshake_filter(deshake)
    ffmpeg_cmd_helper.set_unsharp_filter(unsharp)

    ffmpeg_cmd_helper.execute()


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
]



# src\file_conversor\utils\bulma_utils\audio_video.py

from typing import Any

# user-provided modules
from file_conversor.backend.audio_video.ffmpeg_backend import FFmpegBackend

from file_conversor.utils.dominate_bulma.form_field import FormFieldHorizontal
from file_conversor.utils.dominate_bulma.form_input import FormFieldInput
from file_conversor.utils.dominate_bulma.form_select import FormFieldSelect
from file_conversor.utils.dominate_bulma.form_checkbox import FormFieldCheckbox

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


def TargetSizeField():
    """Create a form field for target file size input."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr="Number.parseInt(value) >= 0",
            current_value="0",
            _name="target-size",
            _type="number",
            help=_("Set target file size (in MB) for the output video. Type 0 to skip size targeting (use encoding speed and quality options to find file size)."),
        ),
        label_text=_("Target Size (MB)"),
    )


def AudioBitrateField():
    """Create a form field for audio bitrate input."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr="Number.parseInt(value) >= 0",
            current_value=CONFIG['audio-bitrate'],
            _name="audio-bitrate",
            _type="number",
            help=_("Set audio bitrate (in kbps). Type 0 to keep original audio bitrate."),
        ),
        label_text=_("Audio Bitrate (kbps)"),
    )


def VideoBitrateField():
    """Create a form field for video bitrate input."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr="Number.parseInt(value) >= 0",
            current_value=CONFIG['video-bitrate'],
            _name="video-bitrate",
            _type="number",
            help=_("Set video bitrate (in kbps). Type 0 to keep original video bitrate."),
        ),
        label_text=_("Video Bitrate (kbps)"),
    )


def AudioCodecField():
    """Create a form field for audio codec selection."""
    return FormFieldHorizontal(
        FormFieldSelect(
            ('', _('Default')),
            *[
                (codec, codec.upper())
                for codec in FFmpegBackend.get_supported_audio_codecs()
            ],
            current_value="",
            _name="audio-codec",
            help=_("Select the audio codec to use for encoding. Not all codecs are supported for all formats."),
        ),
        label_text=_("Audio Codec"),
    )


def VideoCodecField():
    """Create a form field for video codec selection."""
    return FormFieldHorizontal(
        FormFieldSelect(
            ('', _('Default')),
            *[
                (codec, codec.upper())
                for codec in FFmpegBackend.get_supported_video_codecs()
            ],
            current_value="",
            _name="video-codec",
            help=_("Select the video codec to use for encoding. Not all codecs are supported for all formats."),
        ),
        label_text=_("Video Codec"),
    )


def VideoEncodingSpeedField():
    """Create a form field for video encoding speed selection."""
    return FormFieldHorizontal(
        FormFieldSelect(
            *[
                (speed, speed.upper())
                for speed in FFmpegBackend.ENCODING_SPEEDS
            ],
            current_value=CONFIG['video-encoding-speed'],
            _name="video-encoding-speed",
            help=_("Select the video encoding speed. Faster speeds result in lower quality and larger file sizes."),
        ),
        label_text=_("Video Encoding Speed"),
    )


def VideoQualityField():
    """Create a form field for video quality selection."""
    return FormFieldHorizontal(
        FormFieldSelect(
            *[
                (quality, quality.upper())
                for quality in FFmpegBackend.QUALITY_PRESETS
            ],
            current_value=CONFIG['video-quality'],
            _name="video-quality",
            help=_("Select the video quality preset. Higher quality results in larger file sizes."),
        ),
        label_text=_("Video Quality"),
    )


def ResolutionField():
    """Create a form field for video resolution input."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr="/^\\d+:\\d+$/.test(value) || value === ''",
            current_value="",
            _name="resolution",
            _type="text",
            help=_("Set video resolution (e.g., 1920:1080). Leave empty to keep original resolution."),
        ),
        label_text=_("Resolution"),
    )


def FPSField():
    """Create a form field for frames per second (FPS) input."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr="Number.parseInt(value) > 0 || value === ''",
            current_value="",
            _name="fps",
            _type="number",
            help=_("Set frames per second (FPS). Leave empty to keep original FPS."),
        ),
        label_text=_("Frames Per Second (FPS)"),
    )


def BrightnessField():
    """Create a form field for brightness adjustment."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr="!isNaN(parseFloat(value))",
            current_value=str(1.0),
            _name="brightness",
            _type="number",
            step="0.1",
            help=_("Adjust brightness level. Default is 1.0. Values >1.0 increase brightness, <1.0 decrease."),
        ),
        label_text=_("Brightness"),
    )


def ContrastField():
    """Create a form field for contrast adjustment."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr="!isNaN(parseFloat(value))",
            current_value=str(1.0),
            _name="contrast",
            _type="number",
            step="0.1",
            help=_("Adjust contrast level. Default is 1.0. Values >1.0 increase contrast, <1.0 decrease."),
        ),
        label_text=_("Contrast"),
    )


def ColorField():
    """Create a form field for color adjustment."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr="!isNaN(parseFloat(value))",
            current_value=str(1.0),
            _name="color",
            _type="number",
            step="0.1",
            help=_("Adjust color saturation level. Default is 1.0. Values >1.0 increase saturation, <1.0 decrease."),
        ),
        label_text=_("Color Saturation"),
    )


def GammaField():
    """Create a form field for gamma adjustment."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr="!isNaN(parseFloat(value))",
            current_value=str(1.0),
            _name="gamma",
            _type="number",
            step="0.1",
            help=_("Adjust gamma level. Default is 1.0. Values >1.0 increase gamma, <1.0 decrease."),
        ),
        label_text=_("Gamma"),
    )


def SharpnessField():
    """Create a form field for sharpness adjustment."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr="!isNaN(parseFloat(value))",
            current_value=str(1.0),
            _name="sharpness",
            _type="number",
            step="0.1",
            help=_("Adjust sharpness level. Default is 1.0. Values >1.0 increase sharpness, <1.0 decrease."),
        ),
        label_text=_("Sharpness"),
    )


def RotationField():
    """Create a form field for video rotation selection."""
    return FormFieldHorizontal(
        FormFieldSelect(
            *[
                (str(angle), f"{angle}°")
                for angle in [-90, 0, 90, 180]
            ],
            current_value=str(0),
            _name="rotation",
            help=_("Select the rotation angle."),
        ),
        label_text=_("Rotation"),
    )


def MirrorAxisField():
    """Create a form field for video/audio/image mirroring axis selection."""
    return FormFieldHorizontal(
        FormFieldSelect(
            *[
                ("", _("No change")),
                ("x", _("Horizontal")),
                ("y", _("Vertical")),
            ],
            current_value="",
            _name="mirror-axis",
            help=_("Select the axis to mirror the video."),
        ),
        label_text=_("Mirror Axis"),
    )


def DeshakeField():
    """Create a form field for deshake option."""
    return FormFieldCheckbox(
        current_value="off",
        _name="deshake",
        help=_("Enable video deshaking to reduce camera shake effects."),
        label_text=_("Deshake"),
    )


def UnsharpField():
    """Create a form field for unsharp option."""
    return FormFieldCheckbox(
        current_value="off",
        _name="unsharp",
        help=_("Enable unsharp filter to enhance video sharpness."),
        label_text=_("Unsharp"),
    )


__all__ = [
    "TargetSizeField",
    "AudioBitrateField",
    "VideoBitrateField",
    "AudioCodecField",
    "VideoCodecField",
    "VideoEncodingSpeedField",
    "VideoQualityField",
    "ResolutionField",
    "FPSField",
    "BrightnessField",
    "ContrastField",
    "ColorField",
    "GammaField",
    "SharpnessField",
    "RotationField",
    "MirrorAxisField",
    "DeshakeField",
    "UnsharpField",
]




# src\file_conversor\cli\video\enhance_cmd.py
import typer

from pathlib import Path
from typing import Annotated, List

from rich import print

# user-provided modules
from file_conversor.backend import FFmpegBackend

from file_conversor.cli.video._ffmpeg_cmd_helper import FFmpegCmdHelper, EXTERNAL_DEPENDENCIES

from file_conversor.cli.video._typer import TRANSFORMATION_PANEL as RICH_HELP_PANEL
from file_conversor.cli.video._typer import COMMAND_NAME, ENHANCE_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import AudioBitrateOption, BrightnessOption, ColorOption, ContrastOption, DeshakeOption, FPSOption, FormatOption, GammaOption, InputFilesArgument, OutputDirOption, ResolutionOption, UnsharpOption, VideoBitrateOption, VideoEncodingSpeedOption, VideoQualityOption
from file_conversor.utils.validators import check_positive_integer, check_video_resolution, is_close, prompt_retry_on_exception

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    # IMG2PDF commands
    for ext in FFmpegBackend.SUPPORTED_IN_VIDEO_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="enhance",
                description="Enhance",
                command=f'cmd.exe /c "{Environment.get_executable()} "{COMMAND_NAME}" "{ENHANCE_NAME}" "%1""',
                icon=str(icons_folder_path / "color.ico"),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


@typer_cmd.command(
    name=ENHANCE_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Enhance video bitrate, resolution, fps, color, brightness, etc.')}
    """,
    epilog=f"""
        **{_('Examples')}:**

        - `file_conversor {COMMAND_NAME} {ENHANCE_NAME} input_file.avi -od D:/Downloads --color 1.20`

        - `file_conversor {COMMAND_NAME} {ENHANCE_NAME} input_file1.mp4 --unsharp`

        - `file_conversor {COMMAND_NAME} {ENHANCE_NAME} input_file.mkv -cl 0.85 -b 1.10`        
    """)
def enhance(
    input_files: Annotated[List[Path], InputFilesArgument(FFmpegBackend.SUPPORTED_IN_VIDEO_FORMATS)],

    file_format: Annotated[str, FormatOption(FFmpegBackend.SUPPORTED_OUT_VIDEO_FORMATS)] = CONFIG["video-format"],

    audio_bitrate: Annotated[int, AudioBitrateOption()] = CONFIG["audio-bitrate"],
    video_bitrate: Annotated[int, VideoBitrateOption()] = CONFIG["video-bitrate"],

    video_encoding_speed: Annotated[str | None, VideoEncodingSpeedOption(FFmpegBackend.ENCODING_SPEEDS)] = CONFIG["video-encoding-speed"],
    video_quality: Annotated[str | None, VideoQualityOption(FFmpegBackend.QUALITY_PRESETS)] = CONFIG["video-quality"],

    resolution: Annotated[str | None, ResolutionOption()] = None,
    fps: Annotated[int | None, FPSOption()] = None,

    color: Annotated[float, ColorOption()] = 1.0,
    brightness: Annotated[float, BrightnessOption()] = 1.0,
    contrast: Annotated[float, ContrastOption()] = 1.0,
    gamma: Annotated[float, GammaOption()] = 1.0,

    deshake: Annotated[bool, DeshakeOption()] = False,
    unsharp: Annotated[bool, UnsharpOption()] = False,

    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    if (not resolution
            and not fps
            and is_close(color, 1.0)
            and is_close(brightness, 1.0)
            and is_close(contrast, 1.0)
            and is_close(gamma, 1.0)
            and not deshake
            and not unsharp):
        resolution = prompt_retry_on_exception(
            text=f"{_("Target Resolution (width:height) [0:0 = do not change video resolution]")}",
            default="0:0", type=str, check_callback=check_video_resolution,
        )
        resolution = None if resolution == "0:0" else resolution

        fps = prompt_retry_on_exception(
            text=f"{_("Target FPS [0 = do not change FPS]")}",
            default=0, type=int, check_callback=check_positive_integer,
        )
        fps = None if fps is None or fps == 0 else fps

        color = prompt_retry_on_exception(
            text=f"{_("Color adjustment (> 1.0 increases color, < 1.0 decreases color)")}",
            default=1.0, type=float,
        )
        brightness = prompt_retry_on_exception(
            text=f"{_("Brightness adjustment (> 1.0 increases brightness, < 1.0 decreases brightness)")}",
            default=1.0, type=float,
        )
        contrast = prompt_retry_on_exception(
            text=f"{_("Contrast adjustment (> 1.0 increases contrast, < 1.0 decreases contrast)")}",
            default=1.0, type=float,
        )
        gamma = prompt_retry_on_exception(
            text=f"{_("Gamma adjustment (> 1.0 increases gamma, < 1.0 decreases gamma)")}",
            default=1.0, type=float,
        )

        deshake = prompt_retry_on_exception(
            text=f"{_('Apply deshake filter?')}",
            default=False, type=bool,
        )
        unsharp = prompt_retry_on_exception(
            text=f"{_('Apply unsharp filter?')}",
            default=False, type=bool,
        )

    ffmpeg_cmd_helper = FFmpegCmdHelper(
        install_deps=CONFIG['install-deps'],
        verbose=STATE["verbose"],
        overwrite_output=STATE["overwrite-output"],
    )

    # Set arguments for FFmpeg command helper
    ffmpeg_cmd_helper.set_input(input_files)
    ffmpeg_cmd_helper.set_output(file_format=file_format, out_stem="_enhanced", output_dir=output_dir)

    ffmpeg_cmd_helper.set_video_settings(encoding_speed=video_encoding_speed, quality=video_quality)
    ffmpeg_cmd_helper.set_bitrate(audio_bitrate=audio_bitrate, video_bitrate=video_bitrate)

    ffmpeg_cmd_helper.set_resolution_filter(resolution)
    ffmpeg_cmd_helper.set_fps_filter(fps)
    ffmpeg_cmd_helper.set_enhancement_filters(
        brightness=brightness,
        contrast=contrast,
        color=color,
        gamma=gamma,
    )
    ffmpeg_cmd_helper.set_deshake_filter(deshake)
    ffmpeg_cmd_helper.set_unsharp_filter(unsharp)

    ffmpeg_cmd_helper.execute()


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
]
